if BatchObjectRenamer_ObjectList == undefined then (
	persistent global BatchObjectRenamer_ObjectList = #()
	persistent global BatchObjectRenamer_LastFileOperated = ""
)

function UpdateListBox lbRenames = (
	if lbRenames != undefined then (
		displayList = #()
		
		for item in BatchObjectRenamer_ObjectList do (
			displayItem = item[1] + " -> " + item[2]
			
			append displayList displayItem
		)
		
		lbRenames.items = displayList
	)
)

function AddRename oldStr newStr lbRenames = (
	exists = false
	
	for item in BatchObjectRenamer_ObjectList do (
		--if item[1] == oldStr and item[2] == newStr then (
		if (stricmp item[1] oldStr) == 0 and (stricmp item[2] newStr) == 0 then (
			exists = true
		)
	)
	
	if not exists then (
		item = #()
		item[1] = oldStr
		item[2] = newStr
		
		append BatchObjectRenamer_ObjectList item
		
		UpdateListBox lbRenames
	)
)

function ObjectByName oName = (
	-- Because getnodebyname or selectbyname don't seem to reliably work
	for o in objects do (
		--if o.name == oName then (
		if (stricmp o.name oName) == 0 then (
			return o
		)
	)
	
	return undefined
)

function BatchRename oldNameIndex newNameIndex = (
	cnt = 0
	
	for item in BatchObjectRenamer_ObjectList do (
		obj = ObjectByName item[oldNameIndex]
		
		if obj != undefined then (
			obj.name = item[newNameIndex]
			
			cnt = cnt + 1
		)
	)
	
	return cnt
)

function SaveRenames filepath = (
	file = createFile filepath
	for item in BatchObjectRenamer_ObjectList do (
		format "%" (item[1] + "\n") to:file
		format "%" (item[2] + "\n") to:file
	)
	
	messageBox ("Saved " + (BatchObjectRenamer_ObjectList.Count as string) + " renames to \"" + filepath + "\".") beep:false
	
	close file
)

function LoadRenames filepath lbRenames = (
	file = openFile filepath mode:"rt"
	
	readingOld = true
	
	oldName = ""
	
	while not (eof file) do (
		fline = readLine file
		
		if readingOld then (
			oldName = fline
		) else (
			AddRename oldName fLine undefined
			
			oldName = ""
		)
		
		readingOld = not readingOld
	)
	
	UpdateListBox lbRenames
	
	close file
	
	messageBox ("Loaded " + (BatchObjectRenamer_ObjectList.Count as string) + " renames from \"" + filepath + "\".") beep:false
)
	
rollout BatchObjectRenamer_UI "Batch Object Renamer" width:450
(
	edittext txtOld "Find:"
	edittext txtNew "Replace:"
	
	listbox lbRenames "Renames" height:11
	
	button btnAdd "Add Rename" width: 100 across:4
	button btnRemove "Remove Selected" width:100 
	button btnRename "Rename Objects" width:100
	button btnUnrename "Reverse Rename" width:100
	
	button btnSave "Save Renames" width:138 across:3
	button btnLoad "Load Renames" width:138
	button btnClear "Clear Renames" width:138
	
	on BatchObjectRenamer_UI open do (
		UpdateListBox lbRenames
	)
	
	on btnAdd pressed do (
		oldStr = txtOld.text
		newStr = txtNew.text
		
		if oldStr.Count > 0 then (
			if newStr.Count > 0 then (
				AddRename oldStr newStr lbRenames
			) else (
				messageBox "\"Replace\" must not be empty." beep:true
			)
		) else (
			messageBox "\"Find\" must not be empty." beep:true
		)
	)
	
	on btnRemove pressed do (
		selInd = lbRenames.selection
		if selInd > 0 then (
			deleteItem BatchObjectRenamer_ObjectList selInd
			UpdateListBox lbRenames
		)
	)
	
	on btnRename pressed do (
		cnt = BatchRename 1 2
		
		messageBox ("Renamed " + (cnt as string) + " of " + (BatchObjectRenamer_ObjectList.Count as string) + " objects.") beep:false
	)
	
	on btnUnrename pressed do (
		cnt = BatchRename 2 1
		
		messageBox ("Reverse-renamed " + (cnt as string) + " of " + (BatchObjectRenamer_ObjectList.Count as string) + " objects.") beep:false
	)	
	
	on btnClear pressed do (
		cnt = BatchObjectRenamer_ObjectList.Count
		
		BatchObjectRenamer_ObjectList = #()
		UpdateListBox lbRenames
		
		messageBox ("Removed " + (cnt as string) + " renames.") beep:false
	)
	
	on btnSave pressed do (
		filename = getSaveFileName filename:BatchObjectRenamer_LastFileOperated types:"Object Rename Data (*.ord)|*.ord|Text File (*.txt)|*.txt|All|*.*"
		
		if filename != undefined then (
			BatchObjectRenamer_LastFileOperated = filename
			
			SaveRenames filename
		)
	)
	
	on btnLoad pressed do (
		filename = getOpenFileName filename:BatchObjectRenamer_LastFileOperated types:"Object Rename Data (*.ord)|*.ord|Text File (*.txt)|*.txt|All|*.*"
		
		if filename != undefined then (
			BatchObjectRenamer_LastFileOperated = filename
			
			LoadRenames filename lbRenames
		)
	)	
)

createDialog BatchObjectRenamer_UI
	
	