-- Adapted for use with Cannonfodder's
-- This tool is decidedly better

data = #()
firstImport = false

-- Helper functions
function getFilename fPath = (
	fName = filenameFromPath fPath
	fLength = fName.count
	
	fName = substring fName 1 (fLength - 4)
	
	return fName
)

function arrayHas haystack needle = (
	if finditem haystack needle == 0 then (
		return false
	)
	
	return true
)

function selectMeshArray meshArray = (
	selectionGrp = #()
	
	for o in Geometry do (
		if classof o == Editable_Mesh then (
			if arrayHas meshArray o then (
				append selectionGrp o
			)
		)
	)
	
	clearselection
	select selectionGrp
)

-- Core
function doImport filePath = (
	if firstImport then (
		ImportFile filePath
		
		firstImport = false
	) else (
		ImportFile filePath #noprompt
	)
	
	local new = #(filePath)
	
	for o in selection do (
		append new o
		
		if o.material != undefined then (
			if isProperty o.material #diffuseMap then (
				-- Rebuild the material
				newmat = StandardMaterial()
				newmat.diffuseMap = o.material.diffuseMap
				o.material = newmat
			)
		)
	)
	
	print new
	
	append data new
)

function doExport dataObj = (
	local fPath = dataObj[1]
	
	local fName = getFilename fPath
	local rawPath = substring fPath 1 (fPath.count - 4)
	
	local outPath = rawPath + ".smd"
	print ("export to " + rawPath + ".smd")
	
	selectMeshArray dataObj

	ExportSMD outPath true true false false false "1" "2"
)

rollout BatchSkyrimPort_UI "Batch Skyrim Port" (
	button btnImport "Import" toolTip:"Select the NIF meshes to import"
	button btnExport "Export" toolTip:"Will export the SMDs in the same directory as the NIFs imported."
	button btnClear "Clear" toolTip:"Clear the memory of the tool"
	button btnStitch "Stitch" toolTip:"Apply a 0.001 spotweld to all meshes on scene"
	button btnSmooth "Smooth" toolTip:"Smooth all the meshes on the scene"
	
	on btnImport pressed do (
		fileDialog = dotNetObject "System.Windows.Forms.OpenFileDialog"
		fileDialog.title = "Select Files to Import"
		fileDialog.Multiselect = true
		
		onClose = fileDialog.showDialog()
		onClose.toString()
		
		if onClose.Equals onClose.OK then (
			files = fileDialog.fileNames

			firstImport = true
			
			for i = 1 to files.count do (
				doImport files[i]
			)
			
			messageBox ("Imported " + (files.count as String) + " meshes.") beep:false
		)
	)
	
	on btnExport pressed do (
		for i =1 to data.count do (
			doExport data[i]
		)
		
		messageBox ("Exported " + (data.count as String) + " meshes.") beep:false
	)
	
	on btnClear pressed do (
		messageBox ("Clearing " + (data.count as String) + " elements.") beep:false
		
		data = #()
	)
	
	on btnStitch pressed do (
		stitched = 0
		for o in Geometry do (
			if classof o == Editable_Mesh then (
				meshOp.weldVertsByThreshold o.mesh o.mesh.verts 0.0001
				stitched = stitched + 1
			)
		)
		
		messageBox ("Stitched " + (stitched as String) + " meshes." ) beep:false
	)
	
	on btnSmooth pressed do (
		smoothed = 0
		
		for obj in Geometry do (
			if classof obj == Editable_Mesh then (
				obj = convertToPoly obj
				
				local numFaces = polyOp.getNumFaces obj
				local faces = (for i = 1 to numFaces collect i) as bitArray
				
				polyop.setFaceSmoothGroup obj faces 1
				
				obj = convertToMesh obj
				
				smoothed = smoothed + 1
			)
		)

		messageBox ("Smoothed " + (smoothed as String) + "meshes." ) beep:false
	)
)

createDialog BatchSkyrimPort_UI