function getMorphChannelByName morphMod morphName = (
	for i = 1 to 100 do (
		if (WM3_MC_GetName morphMod i) == morphName then (
			return i
		)
	)
	
	return -1
)

function getMorphChannelsByName arrayOfMorphMods morphName = (
	returnArray = #()
	
	for morphMod in arrayOfMorphMods do (
		append returnArray (getMorphChannelByName morphMod morphName)
	)
	
	return returnArray
)

function setAllMorphChannels arrayOfMorphMods morphName morphValue = (
	morphChannels = getMorphChannelsByName arrayOfMorphMods morphName
	
	for ind=1 to morphChannels.count do (
		morphInd = morphChannels[ind]
		
		if morphInd >= 0 then (
			WM3_MC_SetValue arrayOfMorphMods[ind] morphInd morphValue
		)
	)
)

function exportSMDArray fileName exportArray = (
	hideAllBut exportArray bKeepBones:true
	
	ExportSMDSilent filename
)

function MorpherExport flexFolder morphMaster progBar progStatus progCount minv maxv = (
	morphObjects = #()
	morphNames = #()
	
	exportMeshes = getCurrentSelection()
			
	geometryCount = ((getCurrentSelection())).count
	
	progDelta = 100.0 / geometryCount
	curProg = 0.0
	curElems = 0
	
	geometryCount = geometryCount as String
	
	progStatus.text = "Scanning scene"
	progCount.text = "0 / " + geometryCount
	
	obj = morphMaster
		
	if (classOf obj == Editable_Mesh) then (
		if (for modi in obj.modifiers where classof modi == Morpher collect modi).count > 0 then (
			append morphObjects obj
			oMorpher = obj.modifiers[#morpher]
			
			-- Get a running tally of how many morphers there are, for the progress bar.
			for i = 1 to 100 do (
				if WM3_MC_IsValid oMorpher i then (
					mName = WM3_MC_GetName oMorpher i
					
					if mName != "- empty -" then (
						if findItem morphNames mName == 0 then
							append morphNames mName
					)
				)
			)
		)
		
		curProg += progDelta
		curElems += 1
		
		progCount.text = (curElems as String) + " / " + geometryCount
		progBar.value = curProg
	)
	
	curElems = 0
	curProg = 0.0
	progCount.text = "0 / 1"
	progStatus.text = "Exporting base SMD"
	progBar.value = 0
	
	exportRefOnly = true
	
	-- Export the current scene state as the base
	exportSMDArray (flexFolder + "\base.smd") exportMeshes
	
	progCount.text = "1 / 1"
	progBar.value = 100
	
	-- Create an array of morpher modifiers
	morphModsArray = #()
	for obj in morphObjects do (append morphModsArray obj.modifiers[#morpher])
	
	-- Operate on only the first object
	morphObjectsSingle = #()
	append morphObjectsSingle morphObjects[1]
	
	for obj in morphObjectsSingle do (
		numMorphs = 0
		oMorpher = obj.modifiers[#morpher]
		
		for i = 1 to 100 do (
			if WM3_MC_GetName oMorpher i != "- empty -" then
				numMorphs += 1
		)
		
		progStatus.text = "Exporting \""+obj.name+"\"."
		progCount.text = "0 / " + (numMorphs as String)
		progBar.value = 0
		
		progDelta = 100.0 / numMorphs
		curElems = 0
		
		for i =1 to 100 do (
			mName = WM3_MC_GetName oMorpher i
			
			if mName != "- empty -" then (
				mValue = WM3_MC_GetValue oMorpher i
				
				print (flexFolder + "/" + mName + ".smd")

				if mValue == minv then ( -- The morph is this maxed.
					setAllMorphChannels morphModsArray mName maxv
					
					exportSMDArray (flexFolder + "/" + mName + ".smd") exportMeshes
					
					setAllMorphChannels morphModsArray mName mValue
				) else if mValue == maxv then ( -- The morph is this minned
					setAllMorphChannels morphModsArray mName minv
					
					exportSMDArray (flexFolder + "/" + mName + ".smd") exportMeshes
					
					setAllMorphChannels morphModsArray mName mValue
				) else ( -- The morph has both a min and a max.
					setAllMorphChannels morphModsArray mName minv
					
					exportSMDArray (flexFolder + "/" + mName + "_min.smd") exportMeshes
					
					setAllMorphChannels morphModsArray mName maxv
					
					exportSMDArray (flexFolder + "/" + mName + "_max.smd") exportMeshes
					
					setAllMorphChannels morphModsArray mName mValue
				)
				
				curProg += progDelta
				curElems += 1
				
				progBar.value = curProg
				progCount.text = (curElems as String) + " / " + (numMorphs as String)
			)
		)
	)
	
	print "Done!"
)

function doctorString str = (
	nStr = ""
	
	for i = 1 to str.count do (
		if str[i] != "\"" then (
			if str[i] == " " then
				append nStr "_"
			else
				append nStr str[i]
		)
	)
	
	return nStr
)
--flexFolder = ""
flexFolder = "C:/Test/MorpherExport/"
morphMaster = ""

rollout MorpherExport_UI "Morpher Export" width:220 (	
	edittext destFolderDisplay "Flex Folder:"
	button destFolderBtn "Select Flex Folder" toolTip:"This is the ROOT folder. The items will be placed in appropriate subfolders."
	
	label spacer ""
	
	edittext minvBox "Minimum Morph Value:"
	edittext maxvBox "Maximum Morph Value:"
	
	label spacer2 ""
	
	edittext morphMasterDisplay "Morph Master:"
	pickbutton morphMasterBtn "Select Morph Master"
	
	label space3 ""
	
	button btnExport "Export Morphs"
	
	label spacer3 ""
	label progStatus "Not Running"
	progressbar progBar value:0
	label progCount "0 / 0"
	
	on destFolderBtn pressed do (
		destFolder = getSavepath caption:"Select the folder to save flexes to." initialDir:flexFolder
		
		if (destFolder == undefined) then return messageBox "ERROR: Destination-folder browser ended prematurely."

		flexFolder = destFolder
		
		destFolderDisplay.text = flexFolder
	)
	
	on morphMasterBtn picked obj do (
		morphMaster = obj
		
		morphMasterDisplay.text = obj.Name
	)
		
	on btnExport pressed do (
		minvFloat = minvBox.text as float
		maxvFloat = maxvBox.text as float
		
		MorpherExport flexFolder morphMaster progBar progStatus progCount minvFloat maxvFloat
	)
)

createDialog MorpherExport_UI
--MorpherExport "hello"