-- Skyrim Morphs Importer
-- by Gmod4ever
--------------------
-- Feel free to modify and redistribute
-- to your heart's content.
-- All I ask is you provide credit where
-- credit is due.
---------------------

function removeSubstring sourceStr subStr = (
	strLength = sourceStr.count
	substrLength = subStr.count
	strStart = findString sourceStr subStr
	
	newStr = sourceStr
	if strStart != undefined then (
		newStr = (substring sourceStr 1 (strStart - 1)) + (substring sourceStr (strStart + substrLength) strLength)
	)
	
	return newStr
)

function getFilename fPath = (
	fName = filenameFromPath fPath
	fLength = fName.count
	
	fName = substring fName 1 (fLength - 4)
	
	return fName
)

function setObjectName obj fName bUnderscores bUppercase = (
	strName = getFilename fName
	
	if bUnderscores then (
		strName = substituteString strName "_" " "
	)
	
	if bUppercase then (
		strName = toUpper strName
	)
	
	obj.name = strName
)

fn RemoveSkeleton obj = (
	maxops.CollapseNodeTo obj 1 off
	
	local toDel = #()
	for o in Geometry do (
		if classof o == BoneGeometry or classof o == Dummy then (
			append toDel o
		)
	)
	
	for o in toDel do (
		delete o
	)
)

rollout SkyrimMorphImport_UI "Skyrim Morph Import" (
	button btnImport "Import" toolTip:"Select the OBJ meshes to import"
	button btnCull "Name Cull" toolTip:"Rename the first selected object to BASE, and remove its original name and underscores from all other selected objects."
	button btnMorpher "Create Morpher" toolTip:"Will apply a Morpher to the first selected object, with the targets being all other selected objects."
	button btnDeleteTargets "Delete Morph Targets" toolTip:"Will delete every selected object, except the first."
	
	checkbox chkReplaceUnderscores "Replace Underscores"
	checkbox chkUppercase "Uppercase Name"
	checkbox chkLastObject "Rename Last Object Instead"
	checkbox chkRemoveSkeleton "Remove Skeleton"
	checkbox chkCreateMorpher "Procedural Morpher"
	
	on btnImport pressed do (
		fileDialog = dotNetObject "System.Windows.Forms.OpenFileDialog"
		fileDialog.title = "Select Files to Import"
		fileDialog.Multiselect = true
		
		onClose = fileDialog.showDialog()
		onClose.toString()
		
		if onClose.Equals onClose.OK then (
			SWstart = timestamp()
			
			files = fileDialog.fileNames

			-- Force the UI to show up for the first file.
			ImportFile files[1]
			
			local IndexToRename = 1
			if chkLastObject.checked then (
				IndexToRename = selection.count
			)
			
			local obj = selection[IndexToRename]
			local oMod = undefined
			
			SetObjectName obj files[1] chkReplaceUnderscores.checked chkUppercase.checked

			if chkRemoveSkeleton.checked then (
				RemoveSkeleton obj
			)
			
			if chkCreateMorpher.checked then (
				oMod = Morpher()
				
				addModifier obj oMod
			)			
			
			-- And then once the proper settings are applied, import the rest without the prompt
			for i = 2 to files.count do (
				ImportFile files[i] #noPrompt
				
				local obj = selection[IndexToRename]
				
				SetObjectName obj files[i] chkReplaceUnderscores.checked chkUppercase.checked
				
				if chkRemoveSkeleton.checked then (
					RemoveSkeleton obj
				)

				if chkCreateMorpher.checked then (
					WM3_MC_BuildFromNode oMod (i-1) obj
					WM3_MC_SetName oMod (i-1) obj.name

					delete obj
				)
			)
			
			SWend = timestamp()
			
			messageBox ("Imported " + (files.count as String) + " meshes, in " + ((SWend - SWstart) as string) + " ms.") beep:false
		)
	)
	
	on btnCull pressed do (
		if selection.count > 1 then (
			cullName = selection[1].name
			
			selection[1].name = "BASE"
			
			for i=2 to selection.count do (
				selection[i].name = removeSubstring selection[i].name cullName
				selection[i].name = removeSubString selection[i].name "_"
			)
			
			messageBox ("Attempted cull on "+(selection.count as String)+" meshes.") beep:false
		) else (
			messageBox "You must have at least two objects selected."
		)
	)
	
	on btnMorpher pressed do (
		if selection.count > 1 then (
			baseObj = selection[1]
			
			morphMod = Morpher()
			morphMod.Autoload_of_targets = 1
			
			addModifier baseObj morphMod
			
			maxCount = amin selection.count 101 -- Only a maximum of 100 channels supported
			
			for i=2 to maxCount do (
				WM3_MC_BuildFromNode morphMod (i-1) selection[i]
				WM3_MC_SetName morphMod (i-1) selection[i].name
			)
			
			WM3_RebuildInternalCache morphMod
			
			resultString = ("Created "+(maxCount as String)+" morphs for "+(baseObj.name))
				
			if selection.count > maxCount then (
				resultString = "NOTICE: Only 100 morphs were created! Too many selected objects! " + resultString
			)
			
			messageBox resultString beep:false
		) else (
			messageBox "You must have at least two objects selected."
		)
	)
	
	on btnDeleteTargets pressed do (
		if selection.count > 1 then (
			selectionCopy = getCurrentSelection()
			
			maxCount = amin selection.count 101 -- Only a maximum of 100 channels supported
			
			for i=2 to maxCount do (
				delete selectionCopy[i]
			)
			
			resultString = ("Deleted "+((maxCount-1) as String)+" objects.")
			
			if selection.count > maxCount then (
				resultString = "NOTICE: Only 100 objects were deleted! Too many selected objects! " + resultString
			)
			
			messageBox resultString beep:False
		) else (
			messageBox "You must have at least two objects selected."
		)
	)
)

createDialog SkyrimMorphImport_UI