-- Globals
if utilObjSaverBoneArray == undefined then (
	global utilObjSaverBoneArray = #()
)

function getObjectInd obj = (
	for i = 1 to utilObjSaverBoneArray.count do (
		elem = utilObjSaverBoneArray[i]
		
		if elem[1] == obj then ( return i )
	)
	
	return -1
)

function isObjectSaved obj = (
	return getObjectInd obj > -1
)

rollout BatchObjectSaver_UI "Batch Object Saver"
(
	-- Buttons
	button btnSelect "Select Saved objects" \
		tooltip:"Select all the currently saved objects."
	button btnSave "Save Selected Objects" \
		tooltip:"Add the selected bones to the 'saved-object' roster."
	button btnUpdate "Update Saved Objects" \
		tooltip:"Update the position and rotation of all saved objects."
	button btnForget "Forget Selected Objects" \
		tooltip:"Remove the selected bones from the 'saved-object' roster."
	button btnClear "Clear Saved Objects" \
		tooltip:"Forget all selected objects"
	button btnRestorePos "Restore Position" \
		tooltip:"Restore the position of every saved object."
	button btnRestoreRot "Restore Rotation" \
		tooltip:"Restore the rotation of every saved object."
	
	-- Controls	
	on btnSelect pressed do (
		nSelection = #()
		for elem in utilObjSaverBoneArray do (
			append nSelection elem[1]
		)
		
		select nSelection
	)
	
	on btnSave pressed do (
		count = 0
		for item in selection do (
			if getObjectInd item < 1 then (
				newItem = #()
				newItem[1] = item
				newItem[2] = item.rotation
				newItem[3] = item.pos
				
				append utilObjSaverBoneArray newItem
				
				count += 1
			)
		)
		
		messageBox ("Saved " + (count as string) + " new objects.") beep:false
	)
	
	on btnUpdate pressed do (
		count = 0
		item = undefined
		for i = 1 to utilObjSaverBoneArray.count do (
			item = utilObjSaverBoneArray[i][1]
			
			utilObjSaverBoneArray[i][2] = item.rotation
			utilObjSaverBoneArray[i][3] = item.pos
			
			count += 1
		)
		
		messageBox ("Updated " + (count as string) + " saved objects.") beep:false
	)
	
	on btnForget pressed do (
		count = 0
		for item in selection do (
			ind = getObjectInd item
			
			if ind > 0 then (
				deleteItem utilObjSaverBoneArray ind
				count += 1
			)
		)
		
		messageBox ("Forgot " + (count as string) + " saved objects.") beep:false
	)
	
	on btnClear pressed do (
		count = utilObjSaverBoneArray.count
		
		utilObjSaverBoneArray = #()
		
		messageBox ("Forgot " + (count as string) + " saved objects.") beep:false
	)
	
	on btnRestorePos pressed do (
		for item in utilObjSaverBoneArray do (
			if isValidObj item[1] then (
				item[1].pos = item[3]
			)
		)
	)	
	
	on btnRestoreRot pressed do (
		for item in utilObjSaverBoneArray do (
			if isValidObj item[1] then (
				item[1].rotation = item[2]
			)
		)
	)	
)

createDialog BatchObjectSaver_UI