function GetRoots bns = (
	parents = #()
	
	for o in bns do (
		if (findItem bns o.parent) == 0 then ( -- Bone's parent isn't in the selection? It's a root
			append parents o
		)
	)
	
	return parents
)

function EnumerateChains root bns basename cnt = (
	root.name = basename + (cnt as String)
	
	cnt = cnt + 1
	
	for child in root.children do (
		cnt = EnumerateChains child bns basename cnt
	)
	
	return cnt
)

function SelectChain root = (
	bns = #()
	
	appendifunique bns root
	
	for o in root.children do (
		appendifunique bns o
		
		join bns (SelectChain o)
	)
	
	return bns
)

rollout BoneEnumerator_UI "Bone Enumerator" (
	edittext txtBaseName "Base Name"
	spinner spnrIndex "Starting Index" range:[1,999,1] type:#integer value:1
	button btnSelectChain "Select Chain"
	button btnEnumerate "Enumerate Selected Chains"
	
	on btnEnumerate pressed do (
		bns = #()
		
		for o in selection do (
			print (classof o)
			if classof o == BoneGeometry then (
				append bns o
			)
		)
		
		roots = GetRoots bns
		
		if roots.count == 1 then (
			EnumerateChains roots[1] bns (txtBaseName.text + "_") spnrIndex.value
		) else (
			rootcnt = 1
			
			for root in roots do (
				bname = txtBaseName.text + "_" + (rootcnt as String) + "_"
				
				EnumerateChains root bns bname spnrIndex.value
				
				rootcnt = rootcnt + 1
			)
		)
		
		messagebox ("Enumerated " + (bns.count as String) + " bones.") beep:false
	)
	
	on btnSelectChain pressed do (
		bns = #()
		
		for o in selection do (
			print (classof o)
			if classof o == BoneGeometry then (
				append bns o
			)
		)
		
		roots = GetRoots bns
		
		selectedBones = #()
		
		for root in roots do (
			join selectedBones (SelectChain root)
		)
		
		select selectedBones
	)
)

createDialog BoneEnumerator_UI