deleteThreshold = 0.51 -- If a face's GAW is lower than this, it will be deleted.

function GetElementsInMesh obj = (
	elems = #()
	numFaces = meshop.getNumFaces obj
	
	allFaces = #{1 ..  numFaces}
	
	while allFaces.isEmpty == false do (
		faceArray = allFaces as array
		
		face = faceArray[1] -- Just always get the first face
		
		elemFaces = meshop.getElementsUsingFace obj #(face)
		
		append elems elemFaces -- Add these faces to the faces array; they are all one element
		allFaces = allFaces - elemFaces -- And remove these faces from the allFaces array; we don't need to check any of them again, since we know they're all in this same element
	)
	
	elems
)

function GetCenterOfMesh obj = (
	bb = nodeLocalBoundingBox obj
	
	center = point3 ((bb[1].x + bb[2].x) / 2) ((bb[1].y + bb[2].y)/2)  ((bb[1].z + bb[2].z) / 2)
	
	center
)

function BitArrayToInteger ba = (
	if ba.numberSet != 1 then (
		return undefined
	)
	
	return (ba as array)[1]
)

function AngleBetweenPoints v1 v2 = (
	acos(dot (normalize v1) (normalize v2))
)

function VectorBetweenPoints v1 v2 = (
	normalize(v2 - v1)
)

function IsFacingAway obj faceInt = (
	norm = GetFaceNormal obj faceInt
	orig = VectorBetweenPoints (meshop.GetFaceCenter obj faceInt) (GetCenterOfMesh obj)
	
	dotp = dot norm orig
	
	dotp < 0
)

function GetAwayWeight obj elem = (
	totalFaces = elem.count as float
	
	numAway = 0.0
	
	for face in elem do (
		if (IsFacingAway obj face) == true then (
			numAway = numAway + 1
		)
	)
	
	numAway / totalFaces
)

function DeleteDoubleFaces obj = (
	elems = GetElementsInMesh obj
	
	doubleFaces = #{}
	
	for elem in elems do (
		elemArr = elem as array
		
		weight = GetAwayWeight obj elemArr
		
		if weight < deleteThreshold then (
			doubleFaces = doubleFaces + elem
		)
	)
	
	-- Taken from DeleteDoubleFaces 5.0, by Borislav Petrov ; http://www.scriptspot.com/bobo
	local ms = mesh_select ()
	local dm = deletemesh ()
	select obj
	max modify mode
	modPanel.addModToSelection ms
	subObjectLevel = 3
	setfaceselection obj ms doubleFaces
	modPanel.addModToSelection dm
)
		