-- This script is written horribly. You should never write scripts like this.

jiggleSets = #(
	--#(#(list of names),label,negative pitch, positive pitch, negative yaw, positive yaw)
	#(#("front","Front","bang","Bang"),"Front",-2,88,-45,45),
	#(#("left","Left"),"Left",-45,45,-2,88),
	#(#("right","Right"),"Right",-45,45,-88,2),
	#(#("back","Back"),"Back",-88,2,-45,45),
	#(#("ponytail","braid"),"Ponytail",-45,45,-45,45)
)

jiggleExplicit = #()

function write f s = ( -- write <file> <string> - writes <string> to <file>
	format "%" (s + "\n") to:f
)

-- Given a name from "list of names", return the jiggle set that name belongs to.
function JiggleSetFromLabel l = (
	for o in jiggleSets do (
		if o[2] == l then (
			return o
		)
	)
	
	return undefined
)

-- Write the jiggle QCI pre-amble
function WriteJigglePreamble f = (
	write f "// CONSTRAINTS:"
	write f "\t// YAW: -(object right) +(object left)"
	write f "\t// PITCH: (-backward) +(forward)\n"
	write f "// Jigglebones"
	write f "$definevariable mass \"2000\""
	write f "$definevariable length \"35\""
	write f "$definevariable stiffness \"180\""
	write f "$definevariable damping \"6\""
)

-- Collect all of the bones with "_jiggle" in their name. These will be jiggleboned.
function CollectJiggleBones = (
	jiggles = #()
	
	for o in objects do (
		if o.name != undefined and findString o.name "_jiggle" != undefined then (
			append jiggles o
		)
	)
	
	return jiggles
)

-- Write a specific bone's jiggle constraints, using the non-bone-variable settings (minus its name)
function WriteJiggleStart f o = (
	write f ("$jigglebone \"" + o.name + "\" {")
	write f "\tis_flexible {"
	write f "\t\tyaw_stiffness $stiffness$"
	write f "\t\tyaw_damping $damping$"
	write f "\t\tpitch_stiffness $stiffness$"
	write f "\t\tpitch_damping $damping$"
	write f "\t\ttip_mass $mass$"
	write f "\t\tlength $length$"
	write f "\t\tangle_constraint 90"
)

-- Write the tail of a bone's jiggle constraints
function WriteJiggleEnd f = (
	write f "\t}"
	write f "}\n"
)

-- Write an explicit bone's jiggle constraints
function WriteJiggleExplicit f b l = (
	local j = JiggleSetFromLabel l
	
	local names = j[1]
	local label = j[2]
	local neg_pitch = j[3]
	local pos_pitch = j[4]
	local neg_yaw = j[5]
	local pos_yaw = j[6]
	
	write f ("\n//" + label)
	
	for o in b do (
		writejiggleStart f o
			
		write f ("\t\tpitch_constraint " + (neg_pitch as string) + " " + (pos_pitch as string))
		write f ("\t\tyaw_constraint " + (neg_yaw as string) + " " + (pos_yaw as string))
				
		writeJiggleEnd f
	)
)

-- Write all of the explicit jigglebones
function WriteJigglesExplicit f = (
	for ex in jiggleExplicit do (
		local b = ex[1]
		local l = ex[2]
		
		WriteJiggleExplicit f b l
	)
)

-- Automatically grab jigglebones and deduce their jiggle constraints from their names, and write them to file.
function WriteJiggles f b = (
	for j in jiggleSets do (
		local names = j[1]
		local label = j[2]
		local neg_pitch = j[3]
		local pos_pitch = j[4]
		local neg_yaw = j[5]
		local pos_yaw = j[6]
		
		write f ("\n//" + label)
		
		for o in b do (
			hasName = false
			for n in names do (
				if findString o.name n != undefined then (
					hasName = true
					break
				)
			)
			
			if hasName then (
				writejiggleStart f o
				
				write f ("\t\tpitch_constraint " + (neg_pitch as string) + " " + (pos_pitch as string))
				write f ("\t\tyaw_constraint " + (neg_yaw as string) + " " + (pos_yaw as string))
					
				writeJiggleEnd f
			)
		)
	)
)

-- The UI element for selecting a jiggle set label, for explicit jigglebones.
rollout JiggleQCIGenerator_UI_SelectLabel "Jiggle QCI Generator - Select Label" (
	listbox lbLabels "Labels" height:11
	button btnSelect "Select"
	button btnSelectJiggle "Select Only Jiggles"
	
	on JiggleQCIGenerator_UI_SelectLabel open do (
		local labels = #()
		for o in jiggleSets do (
			local label = o[2]
			
			append labels label
		)
		
		lbLabels.items = labels
	)
	
	on btnSelect pressed do (
		selLabel = lbLabels.selected
		newEntry = #(#(),selLabel)
		
		for o in selection do (
			append newEntry[1] o
		)
		
		append jiggleExplicit newEntry
		
		messageBox ("Added " + (newEntry[1].count as string) + " bones explicitly for label \""+ selLabel + "\"") beep:false
	
		DestroyDialog JiggleQCIGenerator_UI_SelectLabel	
	)
	
	on btnSelectJiggle pressed do (
		selLabel = lbLabels.selected
		newEntry = #(#(),selLabel)
		
		for o in selection do (
			if (findstring o.name "_jiggle") != undefined then (
				append newEntry[1] o
			)
		)
		
		append jiggleExplicit newEntry
		
		messageBox ("Added " + (newEntry[1].count as string) + " bones explicitly for label \""+ selLabel + "\"") beep:false
	
		DestroyDialog JiggleQCIGenerator_UI_SelectLabel	
	)
)

-- The UI element for the base jiggle generator.
rollout JiggleQCIGenerator_UI "Jiggle QCI Generator"
(
	button btnSave "Generate QCI"
	button btnExplicit "Mark Selected Bones Explicitly"
	button btnSExplicit "Generate QCI for Explicit Bones"
	
	on btnSave pressed do (
		filename = getSaveFileName types:"QCI (*.qci)|*.qci|Text File (*.txt)|*.txt|All|*.*"
		
		if filename != undefined then (
			fi = createFile filename
			
			WriteJigglePreamble fi

			jBones = CollectJiggleBones()

			WriteJiggles fi jBones

			WriteJigglesExplicit fi
			
			close fi
			
			messageBox ("Wrote " + (jBones.count as string) + " jigglebones to " + filename + ".") beep:false
		)
	)
	
	on btnExplicit pressed do (
		createDialog JiggleQCIGenerator_UI_SelectLabel
	)
	
	on btnSExplicit pressed do (
		filename = getSaveFileName types:"QCI (*.qci)|*.qci|Text File (*.txt)|*.txt|All|*.*"
		
		if filename != undefined then (
			fi = createFile filename
			
			WriteJigglePreamble fi

			WriteJigglesExplicit fi
			
			close fi
			
			messageBox ("Wrote explicit jigglebones to " + filename + ".") beep:false
		)
	)
)

-- Launch the base UI
createDialog JiggleQCIGenerator_UI