
-- EMorph:
	-- String Name
	-- Morpher object
	-- Array of Morphs:
		-- Morph index
		-- Morph value

-- MorphObj:
	-- String object name
	-- Morpher modifier object
	-- #(min,max)
	-- Array of MorphDats:
		-- String morph name
		-- Morph index
	
persistent global ME_LastPresetPath
persistent global ME_LastExportPath
	
if ME_LastPresetPath == undefined then (
	ME_LastPresetPath = ""
)

if ME_LastExportPath == undefined then (
	ME_LastExportPath = ""
)
	
struct MorphDat (
	MorphName, -- String describing the name of the morph
	MorphModIndex -- Int mapping into a Morph modifier
)

struct MorphObj (
	ObjName, -- String describing the object's name
	oMorpher, -- Morpher modifier
	MinMax, -- Point2 describing the morph's range
	MorphData -- Array of MorphDat
)

struct MorphVal (
	MorphObjIndex, -- Int mapping into a MorphObj struct's MorphData field
	MorphValue -- Float describing its value
)

struct EMorph (
	EMorphName, -- String describing the exported-SMD's name
	MObj, -- The MorphObj containing the morph data
	MorphVals -- Array of MorphVals
)

ME_ExportMorphs = undefined
ME_MorphObjects = undefined
ME_MorphValues = undefined
ME_MorphNames = undefined
ME_MorphValue = undefined

ME_DExportMorphs = #()
ME_DMorphObjs = #()

ME_CurEMorphIndex = 0
ME_CurMorphObjIndex = 0

ME_OperatingMorphObject = undefined
ME_PresetOperation = 0 -- 0 = save; 1 = load; 2 = export

fn GetIndexFromName arr sName = (
	for i=1 to arr.count do (
		if arr[i] == sName then (
			return i
		)
	)
	
	return undefined
)

fn PopulateMorphObjects = (
	local MorphObjects = #()
	
	-- Build the data structures
	for o in Geometry do (
		if o.modifiers[#morpher] != undefined then (
			local oMorph = o.modifiers[#morpher]
			local oName = o.name
			
			local mObj = MorphObj ObjName:oName oMorpher:oMorph MinMax:(Point2 oMorph.Spinner_Minimum oMorph.Spinner_Maximum) MorphData:#()
			
			for i=1 to 100 do (
				if (WM3_MC_HasData oMorph i) or (WM3_MC_HasTarget oMorph i) then (
					append mObj.MorphData (MorphDat MorphName:(WM3_MC_GetName oMorph i) MorphModIndex:i)
				)
			)
			
			append MorphObjects mObj
		)
	)
	
	-- Populate the morph-objects dropdown
	ME_MorphObjects.items = (for o in MorphObjects collect o.ObjName)
	ME_DMorphObjs = MorphObjects
	
	MorphObjects
)

fn CreateNewEMorph strName = (
	--local eMorph = #(strName, undefined, #())
	local eMorphObj = EMorph EMorphName:strName MObj:undefined MorphVals:#()
		
	append ME_DExportMorphs eMorphObj
		
	ME_ExportMorphs.items = (for o in ME_DExportMorphs collect o.EMorphName)
		
	if (ME_DExportMorphs.count == 1) then (
		ME_ExportMorphs.selection = 1
	)
	
	return ME_DExportMorphs.count -- Return the index of this new EMorph
)

fn AddMorphToEMorph EMorphInd morphIndex mValue = (
	local mVal = MorphVal MorphObjIndex:morphIndex MorphValue:mValue
	--local morphDat = #(morphIndex, morphValue)
	
	append ME_DExportMorphs[EMorphInd].MorphVals mVal
	
	if EMorphInd == ME_CurEMorphIndex then (
		UpdateMorphValues ME_CurEMorphIndex
	)
)

fn UpdateMorphValues EMorphIndex = (
	morphValues = #()
	
	local EMorph = ME_DExportMorphs[EMorphIndex]
	local mObj = EMorph.MObj
	
	local mVals = EMorph.MorphVals
	
	for morphVal in mVals do (
		append morphValues ((mObj.MorphData[morphVal.MorphObjIndex].MorphName) + " = " + (morphVal.MorphValue as string))
	)
	
	ME_MorphValues.items = morphValues
)

fn UpdateMorphNames drpMorphName spnMorphValue mObj = (
	--local morphObj = ME_DMorphObjs[ME_CurMorphObjIndex]
	
	drpMorphName.items = (for o in mObj.MorphData collect o.MorphName)
	spnMorphValue.range = Point3 mObj.MinMax.x mObj.MinMax.y mObj.MinMax.x
	
	mObj
)	

fn GetMorphObjectFromName sName = (
	for o in ME_DMorphObjs do (
		if o.ObjName == sName then (
			return o
		)
	)
	
	return undefined
)

fn AutoGenerateMorphs = (
	-- First, populate the morph objects
	PopulateMorphObjects()
	
	-- Then loop through everything with a Morpher
	for o in Geometry do (
		if o.modifiers[#morpher] != undefined then (
			local oMorph = o.modifiers[#morpher]
			
			-- Get this object's MorpherObj
			local MObject = GetMorphObjectFromName o.name
			
			-- Generate the indices that will be used in the drop-down for it, for consistency
			local MDropdown = (for o in MObject.MorphData collect o.MorphName)
			
			-- And then build export-morphs for every morph.
			for i=1 to 100 do (
				if (WM3_MC_HasData oMorph i) or (WM3_MC_HasTarget oMorph i) then (
					local morphName = WM3_MC_GetName oMorph i
					
					-- Create and grap the export morph
					local EMInd = CreateNewEMorph morphName
					local EMorphObj = ME_DExportMorphs[EMind]
					
					-- Assign its object morph
					EMorphObj.MObj = MObject
					
					-- Get this morph's index
					local MInd = GetIndexFromName MDropdown morphName
					
					-- And add this morph, at 100% value, to this EMorph
					AddMorphToEMorph EMInd MInd 100.0
				)
			)
		)
	)
)

fn GetEMorphsUsingMorphObj mObj = (
	local EMs = #()
	
	for o in ME_DExportMorphs do (
		if o.MObj == mObj then (
			append EMs o
		)
	)
	
	return EMs
)


fn GetMorphIndexFromName mObj mName = (
	for i=1 to mObj.MorphData.count do (
		if mObj.MorphData[i].MorphName == mName then (
			return i
		)
	)
	
	return undefined
)

fn GetObjectFromMorphObjectName mName = (
	for o in Geometry do (
		if o.name == mName then (
			return o
		)
	)
	
	return undefined
)

fn SavePreset fName EMs = (
	f = createFile fName
	
	-- First, write how many export morphs are in this preset
	format "%\n" (EMs.count) to:f
	
	-- Then, write each of them
	for o in EMs do (
		format "%\n" (o.EMorphName) to:f
		format "%\n" (o.MorphVals.Count as string) to:f
		for mv in o.MorphVals do (
			format "%\n" o.MObj.MorphData[mv.MorphObjIndex].MorphName to:f
			format "%\n" (mv.MorphValue as string) to:f
		)
	)
	
	messagebox ("Saved " + (EMs.count as string) + " export-morphs to preset \"" + (fName) + "\".") beep:false
	
	close f
)

fn DoSavePreset = (
	local EMs = GetEMorphsUsingMorphObj ME_OperatingMorphObject
	
	if EMs.count > 0 then (	
		local fName = getSaveFileName filename:ME_LastPresetPath  types:"Morph Exporter Preset (*.mep)|*.mep|Text File (*.txt)|*.txt|All|*.*"
		
		if fName != undefined then (
			ME_LastPresetPath = fName
			
			SavePreset fName EMs
		)
	) else (
		messagebox "No export morphs use the selected Morph Object." beep:true
	)
)

fn LoadPreset fName mObj = (
	f = openFile fName
	
	local numExports = (readLine f) as number
	
	local total = 0
	local successes = 0
	
	for i=1 to numExports do (
		local MEname = readLine f
		
		local MEInd = CreateNewEMorph MEname
		local ME = ME_DExportMorphs[MEInd]
		
		ME.MObj = mObj
		
		local numMorphs = (readLine f) as number
		
		for j=1 to numMorphs do (
			local mName = readLine f
			local mVal = (readLine f) as float
			
			local mInd = GetMorphIndexFromName mObj mName
			
			total = total + 1
			
			if mInd != undefined then (
				AddMorphToEMorph MEInd (GetMorphIndexFromName mObj mName) mVal
				
				successes = successes + 1
			)
		)
	)

	ME_ExportMorphs.items = (for o in ME_DExportMorphs collect o.EMorphName)
	
	messageBox ("Mapped " + (successes as string) + " of " + (total as string) + " morphs, across " + (numExports as string) + " export-morphs, onto \"" + (mObj.ObjName) + "\".") beep:false
)

fn DoLoadPreset = (
	fName = getOpenFileName filename:ME_LastPresetPath types:"Morph Exporter Preset (*.mep)|*.mep|Text File (*.txt)|*.txt|All|*.*"
	
	if fName != undefined then (
		ME_LastPresetPath = fName
		
		LoadPreset fName ME_OperatingMorphObject
	)
)

-- struct MorphDat (
-- 	MorphName, -- String describing the name of the morph
-- 	MorphModIndex -- Int mapping into a Morph modifier
-- )

-- struct MorphObj (
-- 	ObjName, -- String describing the object's name
-- 	oMorpher, -- Morpher modifier
-- 	MinMax, -- Point2 describing the morph's range
-- 	MorphData -- Array of MorphDat
-- )

-- struct MorphVal (
-- 	MorphObjIndex, -- Int mapping into a MorphObj struct's MorphData field
-- 	MorphValue -- Float describing its value
-- )

-- struct EMorph (
-- 	EMorphName, -- String describing the exported-SMD's name
-- 	MObj, -- The MorphObj containing the morph data
-- 	MorphVals -- Array of MorphVals
-- )

fn ExportMesh fName obj = (
	local eGrps = #()
	
	append eGrps (createExportGroup fName #(obj))
		
	ExportSMDSilentBatch eGrps bKeepBones:false
)

fn ExportMorphs fPath mObj = (
	local eObj = GetObjectFromMorphObjectName mObj.ObjName
	local EMs = GetEMorphsUsingMorphObj mObj
	
	-- First, zero out all of the morphs
	local mMod = mObj.oMorpher

	for i=1 to 100 do (
		WM3_MC_SetValue mMod i 0.0
	)	
	
	-- Next, export the base frame
	local baseName = fPath + "/base.smd"
	
	ExportMesh baseName eObj
	--print ("ExportMesh \"" + (baseName) + "\" \"" + (eObj as string) + "\"")
	
	-- Then go through all of the export morphs, and export them all out.
	for EM in EMs do (
		local EMName = EM.EMorphName
					
		for MV in EM.MorphVals do (
			local mInd = mObj.MorphData[MV.MorphObjIndex].MorphModIndex
			local mVal = MV.MorphValue
			
			WM3_MC_SetValue mMod mInd mVal
			
			local eName = fPath + "/" + EMName + ".smd"
			
			ExportMesh eName eObj
			--print ("ExportMesh \"" + (eName) + "\" \"" + (eObj as string) + "\"")
			
			WM3_MC_SetValue mMod mInd 0.0 -- Set the value back to 0 before the next morph is exported.

		)
	)
)

fn DoExportMorphs = (
	local EMs = GetEMorphsUsingMorphObj ME_OperatingMorphObject
	
	if EMs.count > 0 then (
		ExportMorphs ME_LastExportPath ME_OperatingMorphObject
	) else (
		messagebox "There are no export morphs using the selected Morph Object." beep:true
	)
)

rollout MorphExport_UI_ChooseMorphObject "Morph Exporter - Choose Morph Object" (
	listbox lstMorphObjs "Select Morph Object:" width:140
	button btnSelect "Continue" width:60 enabled:false
	
	on MorphExport_UI_ChooseMorphObject open do (		
		lstMorphObjs.items = (for o in ME_DMorphObjs collect o.ObjName)
	)
	
	on lstMorphObjs selected itmIndex do (
		btnSelect.enabled = true
	)
	
	on btnSelect pressed do (
		ME_OperatingMorphObject = ME_DMorphObjs[lstMorphObjs.selection]
		
		if ME_PresetOperation == 0 then (
			DoSavePreset()
		)
		if ME_PresetOperation == 1 then (
			DoLoadPreset()
		)
		if ME_PresetOperation == 2 then (
			DoExportMorphs()
		)
		
		DestroyDialog MorphExport_UI_ChooseMorphObject
	)
)

rollout MorphExport_UI "Morph Exporter" width:440 (
	-- UI Generation
	edittext txtExportPath "Export Path: " pos:[8,8] width:300 height:20
	button btnPickExport "Browse ..." pos:[316,8] width:120
	
	edittext txtEMorphName "Name: " pos:[8,40] width:140 height:20
	listbox lstExportMorphs "Export Morphs:" pos:[8,66] width:140 height:14
	button btnAddEMorph "+" pos:[8,275] width:24 enabled:false
	button btnRemoveEMorph "-" pos:[36,275] width:24 enabled:false
	button btnAutoEMorph "Auto" pos:[64,275] width:48
	
	dropdownlist drpMorphObject "Morph Object:" pos:[152,40] width:160 enabled:false
	listbox lstMorphValues "Morphs:" pos:[152,92] width:160 height:12 enabled:false
	button btnAddMorphValue "+" pos:[152,275] width:24 enabled:false
	button btnRemoveMorphValue "-" pos:[180,275] width:24 enabled:false
	
	dropdownlist drpMorphName "Morph Name:" pos:[316,40] width:120 enabled:false
	spinner spnMorphValue "Morph Value" pos:[360,92] width:70 enabled:false
	
	button btnRename "Rename Export Morph" pos:[316,120] width:120 enabled:false
	button btnLoadPreset "Load Preset" pos:[316,146] width:120
	button btnSavePreset "Save Preset" pos:[316,172] width:120 enabled:false
	button btnExport "Export" pos:[316,250] width:120 height:48 enabled:false
	
	-- Rollout handlers
	on MorphExport_UI open do (
		ME_ExportMorphs = lstExportMorphs
		ME_MorphObjects = drpMorphObject
		ME_MorphValues = lstMorphValues
		ME_MorphName = drpMorphName
		ME_MorphValue = spnMorphValue
		
		txtExportPath.text = ME_LastExportPath
	)
	
	-- Export-path controls
	on txtExportPath changed txt do (
		ME_LastExportPath =  txt
	)
	
	on btnPickExport pressed do (
		local fPath = GetSavepath initialDir:ME_LastExportPath
		
		if fPath != undefined then (
			ME_LastExportPath = fPath
			
			txtExportPath.text = fPath
		)
	)
	
	-- EMorph handlers
	on lstExportMorphs selected itmInd do (
		ME_CurEMorphIndex = itmInd
		
		btnRemoveEMorph.enabled = true
		
		lstMorphvalues.enabled = true
		
		drpMorphName.selection = 1
		
		if ME_DMorphObjs.count < 1 then (
			PopulateMorphObjects()
		)
		
		if txtEMorphName.text.count > 0 then (
			btnRename.enabled = true
		)
		
		if ME_DExportMorphs[itmInd].MObj != undefined then (
			drpMorphObject.enabled = false
			btnAddMorphValue.enabled = true
			
			drpMorphObject.selection = GetIndexFromName drpMorphObject.items ME_DExportMorphs[itmInd].MObj.ObjName
			
			UpdateMorphValues ME_CurEMorphIndex
			
			UpdateMorphNames drpMorphName spnMorphValue ME_DExportMorphs[itmInd].MObj
		) else (
			lstMorphValues.items = #()
			
			drpMorphObject.enabled = true
			btnAddMorphValue.enabled = false
			btnRemoveMorphValue.enabled = false	

			drpMorphObject.selection = 0
			drpMorphName.selection = 0
		)		
	)
	
	on txtEMorphName changed txt do (
		if txt.count > 0 then (
			btnAddEMorph.enabled = true
			
			if lstExportMorphs.selection > 0 then (
				btnRename.enabled = true
			)
		) else (
			btnAddEMorph.enabled = false
			btnRename.enabled = false
		)
	)
	
	on btnAddEMorph pressed do (
		CreateNewEMorph txtEMorphName.text
		
		btnAddEMorph.enabled = false
		btnSavePreset.enabled = true
		btnExport.enabled = true
		btnRename.enabled = false
		
		txtEMorphName.text = ""
		
	)
	
	on btnRemoveEMorph pressed do (
		deleteItem ME_DExportMorphs ME_ExportMorphs.selection
		
		ME_ExportMorphs.items = (for o in ME_DExportMorphs collect o.EMorphName)
		ME_MorphValues.items = #()
		
		btnAddMorphValue.enabled = false
		btnRemoveMorphValue.enabled = false	

		if (ME_ExportMorphs.items.count < 1) or (ME_ExportMorphs.selection < 1) then (
			btnRemoveEMorph.enabled = false
			btnSavePreset.enabled = false
			btnExport.enabled = false
		)
	)
	
	on btnAutoEMorph pressed do (
		AutoGenerateMorphs()
		
		btnSavePreset.enabled = true
	)
	
	-- MorphObj handlers	
	on drpMorphObject selected itmIndex do (
		ME_CurMorphObjIndex = itmIndex
		
		drpMorphName.enabled = true
		spnMorphValue.enabled = true
		btnAddMorphValue.enabled = true
		btnRemoveMorphValue.enabled = false	
				
		drpMorphObject.enabled = false
		
		local mObj = UpdateMorphNames drpMorphName spnMorphValue ME_DMorphObjs[ME_CurMorphObjIndex]
		
		ME_DExportMorphs[ME_CurEMorphIndex].MObj = mObj
	)
	
	on lstMorphValues selected itmIndex do (
		btnRemoveMorphValue.enabled = true
	)
	
	on btnAddMorphValue pressed do (
		AddMorphToEMorph ME_CurEMorphIndex drpMorphName.selection spnMorphValue.value
	)
	
	on btnRemoveMorphValue pressed do (
		local EMObj = ME_DExportMorphs[ME_CurEMorphIndex]
		
		local mVals = EMObj.MorphVals
		
		deleteItem mVals ME_MorphValues.selection
		
		EMObj.MorphVals = mVals
		
		UpdateMorphValues ME_CurEMorphIndex
		
		if (mVals.count < 1) or (ME_MorphValues.selection < 1) then (
			btnRemoveMorphValue.enabled = false
		)
	)

	-- General controls
	on btnRename pressed do (
		ME_DExportMorphs[ME_CurEMorphIndex].EMorphName = txtEMorphName.text
		
		ME_ExportMorphs.items = (for o in ME_DExportMorphs collect o.EMorphName)
		
		btnAddEMorph.enabled = false
		btnRename.enabled = false
		
		txtEMorphName.text = ""
	)
	
	on btnLoadPreset pressed do (
		if ME_DMorphObjs.Count < 1 then (
			PopulateMorphObjects()
		)
		
		if ME_DMorphObjs.count == 1 then (
			ME_OperatingMorphObject = ME_DMorphObjs[1]
			
			DoLoadPreset()
		) else (
			ME_PresetOperation = 1
			
			createDialog MorphExport_UI_ChooseMorphObject
		)
		
		btnSavePreset.enabled = true
		btnExport.enabled = true
	)
	
	on btnSavePreset pressed do (
		if ME_DMorphObjs.Count < 1 then (
			PopulateMorphObjects()
		)
		
		if ME_DMorphObjs.count == 1 then (
			ME_OperatingMorphObject = ME_DMorphObjs[1]
			
			DoSavePreset()
		) else (
			ME_PresetOperation = 0
			
			createDialog MorphExport_UI_ChooseMorphObject
		)
	)
	
	on btnExport pressed do (
		if ME_LastExportPath.count > 0 then (
			if ME_DMorphObjs.Count < 1 then (
				PopulateMorphObjects()
			)
			
			if ME_DMorphObjs.count == 1 then (
				ME_OperatingMorphObject = ME_DMorphObjs[1]
				
				DoSavePreset()
			) else (
				ME_PresetOperation = 2
				
				createDialog MorphExport_UI_ChooseMorphObject
			)
		) else (
			messagebox "No export path selected." beep:true
		)
	)
)

createDialog MorphExport_UI