PG_StartingIndex = 1
PG_Range = 10

PG_SliderLookup = #()
PG_Phonemes = #()

struct PG_PhoneMorph (morphName, morphIndex, maxValue)
struct PG_Phoneme (name, curValue, phoneMorphs)

PG_EmptyPhoneme = PG_Phoneme name:"- empty -" curValue:0.00 phoneMorphs:#()

fn GetPhonemeData arrInd = (
	if PG_Phonemes[arrInd] == undefined then (
		return PG_EmptyPhoneme
	)
	
	return PG_Phonemes[arrInd]
)

fn UpdatePhonemeMorphs arrInd = (
	-- Stub function
)

fn OnPhonemeSliderUpdate arrInd val valLbl = (
	local data = GetPhonemeData arrInd
	
	valLbl.text = (val as string)
	
	if data != PG_EmptyPhoneme then (
		PG_Phonemes[arrInd].curValue = val
		
		UpdatePhonemeMorphs arrInd
	)	
)

fn PopulateSliderLookup = (
	PG_SliderLookup[1] = #(PhonemeGenerator_UI.lblPhoneme1Name, PhonemeGenerator_UI.lblPhoneme1Value, PhonemeGenerator_UI.sldPhoneme1)
	PG_SliderLookup[2] = #(PhonemeGenerator_UI.lblPhoneme2Name, PhonemeGenerator_UI.lblPhoneme2Value, PhonemeGenerator_UI.sldPhoneme2)
	PG_SliderLookup[3] = #(PhonemeGenerator_UI.lblPhoneme3Name, PhonemeGenerator_UI.lblPhoneme3Value, PhonemeGenerator_UI.sldPhoneme3)
	PG_SliderLookup[4] = #(PhonemeGenerator_UI.lblPhoneme4Name, PhonemeGenerator_UI.lblPhoneme4Value, PhonemeGenerator_UI.sldPhoneme4)
	PG_SliderLookup[5] = #(PhonemeGenerator_UI.lblPhoneme5Name, PhonemeGenerator_UI.lblPhoneme5Value, PhonemeGenerator_UI.sldPhoneme5)
	PG_SliderLookup[6] = #(PhonemeGenerator_UI.lblPhoneme6Name, PhonemeGenerator_UI.lblPhoneme6Value, PhonemeGenerator_UI.sldPhoneme6)
	PG_SliderLookup[7] = #(PhonemeGenerator_UI.lblPhoneme7Name, PhonemeGenerator_UI.lblPhoneme7Value, PhonemeGenerator_UI.sldPhoneme7)
	PG_SliderLookup[8] = #(PhonemeGenerator_UI.lblPhoneme8Name, PhonemeGenerator_UI.lblPhoneme8Value, PhonemeGenerator_UI.sldPhoneme8)
	PG_SliderLookup[9] = #(PhonemeGenerator_UI.lblPhoneme9Name, PhonemeGenerator_UI.lblPhoneme9Value, PhonemeGenerator_UI.sldPhoneme9)
	PG_SliderLookup[10] = #(PhonemeGenerator_UI.lblPhoneme10Name, PhonemeGenerator_UI.lblPhoneme10Value, PhonemeGenerator_UI.sldPhoneme10)
)

fn UpdatePhonemeSlider arrInd lblName slder lblVal = (
	if ((arrInd >= PG_StartingIndex) and (arrInd < (PG_StartingIndex + PG_Range - 1))) then (
		local data = GetPhonemeData arrInd
		
		lblVal.text = data.curValue as string
		lblName.text = data.name
		slder.value = data.curValue
	)
)

fn UpdateAllPhonemeSliders = (
	local startVal = PG_StartingIndex
	local endVal = startVal + PG_Range - 1
	
	for i=startVal to endVal do (
		local sliderVal = i - startVal + 1
		
		local sliderElements = PG_SliderLookup[sliderVal]
		
		UpdatePhonemeSlider i sliderElements[1] sliderElements[3] sliderElements[2]
	)
	
	-- Also update the range, just in case
	PhonemeGenerator_UI.lblRange.text = "Phonemes " + (startVal as string) + " - " + (endVal as string)
)

fn Test_GenerateGarbagePhonemes numPhonemes = (
	for i=1 to numPhonemes do (
		local phoneme = PG_Phoneme name:("Test" + (i as string)) curValue:0.00 phoneMorphs:#()
		
		append PG_Phonemes phoneme
	)
	
	UpdateAllPhonemeSliders()
)

rollout PhonemeGenerator_UI "Phoneme Generator" width:250 height:410 (
	pickbutton pckPick "Pick Object to Build Phonemes For" width:240 height:20 pos:[5,5]
	button btnLoad "Load Phonemes From File" width:240 height:20 pos:[5,30]
	
	--subrollout sroSliders "Sliders" height:300
	
	-- God I hate not being able to systemically create these...
	
	-- Set the current range text
	label lblRange "Phonemes 1 - 10" height:16 pos:[30,60]
	
	-- And then the sliders. Oh boy.
	label lblPhoneme1Name "- empty -" pos:[20,95] width:60 height:32
	slider sldPhoneme1 "" range:[0,100,0] type:#float orient:#horizontal pos:[75,85] width:145
	label lblPhoneme1Value "0.0" pos:[220,95]
	
	label lblPhoneme2Name "- empty -" pos:[20,125] width:60 height:32
	slider sldPhoneme2 "" range:[0,100,0] type:#float orient:#horizontal pos:[75,115] width:145
	label lblPhoneme2Value "0.0" pos:[220,125]

	label lblPhoneme3Name "- empty -" pos:[20,155] width:60 height:32
	slider sldPhoneme3 "" range:[0,100,0] type:#float orient:#horizontal pos:[75,145] width:145
	label lblPhoneme3Value "0.0" pos:[220,155]

	label lblPhoneme4Name "- empty -" pos:[20,185] width:60 height:32
	slider sldPhoneme4 "" range:[0,100,0] type:#float orient:#horizontal pos:[75,175] width:145
	label lblPhoneme4Value "0.0" pos:[220,185]

	label lblPhoneme5Name "- empty -" pos:[20,215] width:60 height:32
	slider sldPhoneme5 "" range:[0,100,0] type:#float orient:#horizontal pos:[75,205] width:145
	label lblPhoneme5Value "0.0" pos:[220,215]
	
	label lblPhoneme6Name "- empty -" pos:[20,245] width:60 height:32
	slider sldPhoneme6 "" range:[0,100,0] type:#float orient:#horizontal pos:[75,235] width:145
	label lblPhoneme6Value "0.0" pos:[220,245]

	label lblPhoneme7Name "- empty -" pos:[20,275] width:60 height:32
	slider sldPhoneme7 "" range:[0,100,0] type:#float orient:#horizontal pos:[75,265] width:145
	label lblPhoneme7Value "0.0" pos:[220,275]
	
	label lblPhoneme8Name "- empty -" pos:[20,305] width:60 height:32
	slider sldPhoneme8 "" range:[0,100,0] type:#float orient:#horizontal pos:[75,295] width:145
	label lblPhoneme8Value "0.0" pos:[220,305]	
	
	label lblPhoneme9Name "- empty -" pos:[20,335] width:60 height:32
	slider sldPhoneme9 "" range:[0,100,0] type:#float orient:#horizontal pos:[75,325] width:145
	label lblPhoneme9Value "0.0" pos:[220,335]

	label lblPhoneme10Name "- empty -" pos:[20,365] width:60 height:32
	slider sldPhoneme10 "" range:[0,100,0] type:#float orient:#horizontal pos:[75,355] width:145
	label lblPhoneme10Value "0.0" pos:[220,365]
	
	-- Put the up/down buttons down here, so they're on top of the sliders
	button btnUp "^" width:16 height:16 pos:[5,60]
	button btnDown "v" width:16 height:16 pos:[5,385]
	
	-- And the slider events
	on sldPhoneme1 changed val do ( OnPhonemeSliderUpdate 1 val lblPhoneme1Value )
	on sldPhoneme2 changed val do ( OnPhonemeSliderUpdate 2 val lblPhoneme2Value )
	on sldPhoneme3 changed val do ( OnPhonemeSliderUpdate 3 val lblPhoneme3Value )
	on sldPhoneme4 changed val do ( OnPhonemeSliderUpdate 4 val lblPhoneme4Value )
	on sldPhoneme5 changed val do ( OnPhonemeSliderUpdate 5 val lblPhoneme5Value )
	on sldPhoneme6 changed val do ( OnPhonemeSliderUpdate 6 val lblPhoneme6Value )
	on sldPhoneme7 changed val do ( OnPhonemeSliderUpdate 7 val lblPhoneme7Value )
	on sldPhoneme8 changed val do ( OnPhonemeSliderUpdate 8 val lblPhoneme8Value )
	on sldPhoneme9 changed val do ( OnPhonemeSliderUpdate 9 val lblPhoneme9Value )
	on sldPhoneme10 changed val do ( OnPhonemeSliderUpdate 10 val lblPhoneme10Value )
	
	-- The shifter events
	on btnDown pressed do (
		PG_StartingIndex = PG_StartingIndex + 1
		
		UpdateAllPhonemeSliders()
	)
	
	on btnUp pressed do (
		if PG_StartingIndex > 1 then (
			PG_StartingIndex = PG_StartingIndex - 1
			
			UpdateAllPhonemeSliders()
		)
	)
	
	-- The rollout events
	on PhonemeGenerator_UI open do (
		PopulateSliderLookup()
		Test_GenerateGarbagePhonemes 3
		
	)
)

--rollout PhonemeGenerator_SlidersSubroll "Phoneme Sliders" (
--	button btnOne "It's a button"
--	button btnTwo "It's another button"
--	button btnThree "SO MANY BUTTONS"
--)

CreateDialog PhonemeGenerator_UI

--AddSubRollout PhonemeGenerator_UI.sroSliders PhonemeGenerator_SlidersSubroll