-- Core functionality
poseSets = #()
lbox_availPoses = undefined
lbox_ignoreBones = undefined
lbox_transposeBones = undefined
lbox_missingBones = undefined

ignoreBones_Array = #()
transposeBones_Array =  #()
activePoseSet = undefined

function AppendListBox lbox newValue = (
	curItems = lbox.items
	
	append curItems newValue
	
	lbox.items = curItems
)

function RemoveFromListBox lbox index = (
	curItems = lbox.items
	
	deleteItem curItems index
	
	lbox.items = curItems
)

-- write <file> <string> - writes <string> to <file>
function write f s = ( 
	format "%" (s + "\n") to:f
	--print s
)

function SavePoses = (
	scriptsDir = GetDir #scripts
	fname = scriptsDir + "\pose_transposer_data.txt"
	
	f = createFile fname
	
	write f (poseSets.count as string) -- How many posesets there are
	
	for poseset in posesets do (
		write f poseset[1] -- Name
		
		write f ((poseset.count-1) as string) -- How many bones there are
		
		for i=2 to poseset.count do (
			boneinfo = poseset[i]
			
			quatAsString = (boneInfo[2] as string) + " " + (boneInfo[3] as string)  + " " + (boneInfo[4] as string)  + " " + (boneInfo[5] as string) 
			
			write f boneinfo[1] -- Bone name
			write f quatAsString -- Rotation
		)
	)
	
	close f
)

function LoadPoses = (
	scriptsDir = GetDir #scripts
	fname = scriptsDir + "\pose_transposer_data.txt"

	f = openFile fname
	
	if f != undefined then (
		numPoses = (readLine f) as number
		
		for i=1 to numPoses do (
			poseSetName = readLine f
			
			poseSet = #(poseSetName)
			
			numBones = (readLine f) as number
			
			for j=1 to numBones do (
				boneName = readLine f
				quatString = readLine f
				
				rawQuat = filterString quatString " "
				
				boneInfo = #(boneName,rawQuat[1] as number,rawQuat[2] as number,rawQuat[3] as number,rawQuat[4] as number)
				
				append poseSet boneInfo
			)
			
			append poseSets poseSet
			
			AppendListBox lbox_availposes poseSetName
		)
		
		close f
	)
)

function CreatePoseSet poseName bonesInclude = (
	local newPoseSet = #(poseName)
	
	for o in bonesInclude do (
		local newBoneInfo = #(o.name, o.rotation.x, o.rotation.y, o.rotation.z, o.rotation.w)
		
		append newPoseSet newBoneInfo
	)
	
	append poseSets newPoseSet
	
	AppendListBox lbox_availPoses poseName
	
	SavePoses()
)	

function ReverseArray arr = (
	arr = for i=arr.count to 1 by -1 collect arr[i]
	return arr
)
	
function CrawlBoneTree boneToStart bonesToIgnore collectedBones = (
	for o in boneToStart.children do (
		if FindItem bonesToIgnore o == 0 then (
			CrawlBoneTree o bonesToIgnore collectedBones
		)
	)
	
	append collectedBones boneToStart
	
	return collectedBones
)

function GetMissingBones poseBones = (
	missingBones = #()
	
	for oname in poseBones do (
		if getnodebyname oname == undefined then (
			append missingBones oname
		)
	)
	
	return missingBones
)

function LoadPose poseSet = (
	for i=2 to poseSet.count do (
		obone = getnodebyname poseSet[i][1]
		
		pos = obone.position
		
		newrot = quat poseSet[i][2] poseSet[i][3] poseSet[i][4] poseSet[i][5]
		
		obone.rotation = newrot
		obone.pos = pos
	)
)

-- UI
rollout PoseTransposer_Create_UI "Create Pose Transposition" width:220 (
	edittext txtPoseName "Pose Name"
	button btnIgnore "Add Ignore Bone"
	listbox lstIgnoreBones "Ignored Bones"
	button btnRoot "Add Root Bone"
	listbox lstTransposeBones "Transposing Bones"
	button btnCreate "Create Pose" enabled:false
	
	on PoseTransposer_Create_UI open do (
		ignoreBones_Array = #()
		transposeBones_Array = #()
		
		lbox_ignoreBones = lstIgnoreBones
		lbox_transposeBones = lstTransposeBones
	)
	
	on txtPoseName changed txt do (
		if btnroot.enabled == false then (
			btnCreate.enabled = true
		)
	)
	
	on btnIgnore pressed do (
		for o in selection do (
			if classof o == BoneGeometry or classof o == Dummy then (
				append ignoreBones_Array o
				
				AppendListBox lbox_ignoreBones o.name
			)
		)
	)
	
	on btnRoot pressed do (
		rootBone = selection[1]
		
		transposeBones = #()
		CrawlBoneTree rootBone ignoreBones_Array transposeBones
		
		transposeBones = ReverseArray transposeBones		
		
		for o in transposeBones do (
			AppendListBox lbox_transposeBones o.name
		)
		
		transposeBones_Array = transposeBones
		
		btnRoot.enabled = false
		
		if txtPoseName.text.count > 0 then (
			btnCreate.enabled = true
		)
	)
	
	on btnCreate pressed do (
		CreatePoseSet txtPoseName.text transposeBones_Array
		
		DestroyDialog PoseTransposer_Create_UI
	)
)

rollout PoseTransposer_SaveNewPose_UI "Save New Pose" (
	edittext txtPoseName "Pose Name"
	button btnSave "Save Pose" enabled:false
	
	on txtPoseName changed txt do (
		btnSave.enabled = true
	)
	
	on btnSave pressed do (
		activePoseSet[1] = txtPoseName.text
		
		append poseSets activePoseSet
		
		AppendListBox lbox_availposes txtPoseName.text
		
		DestroyDialog PoseTransposer_SaveNewPose_UI
		
		SavePoses()
	)
)

rollout PoseTransposer_FixMissingBones_UI "Fix Missing Bones" width:220 (
	listbox lstMissingBones "Missing Bones"
	edittext txtNewName "Replacement Bone"
	button btnReplace "Replace"
	button btnSave "Save As New Pose" enabled:false
	button btnLoad "Load Pose" enabled:false
	
	on PoseTransposer_FixMissingBones_UI open do (
		lbox_missingBones = lstMissingBones
		
		boneNames = #()
		for i=2 to activePoseSet.count do (
			append boneNames activePoseSet[i][1]
		)
	
		missingBoneNames = GetMissingBones boneNames
		
		for oname in missingBoneNames do (
			AppendListBox lstMissingbones oname
		)
	)
	
	on btnReplace pressed do (
		newBoneName = txtNewName.text
		selectedBoneName = lstMissingBones.selected
		
		newBone = getnodebyname newBoneName
		
		if newBone != undefined then (
			missingBoneIndex = -1
			for i=2 to activePoseSet.count do (
				if activePoseSet[i][1] == selectedBoneName then (
					missingBoneIndex = i
					exit
				)
			)
			
			if missingBoneIndex > 0 then (
				activePoseSet[missingBoneIndex][1] = newBoneName
				RemoveFromListBox lstMissingBones lstMissingBones.selection
				
				if lstMissingBones.items.count < 1 then (
					btnReplace.enabled = false
					btnSave.enabled = true
					btnLoad.enabled = true
				)
			)
		) else (
			messagebox "New bone not found" beep:true
			
			txtNewName.text = ""
		)
	)
	
	on btnLoad pressed do (
		LoadPose activePoseSet
		
		DestroyDialog PoseTransposer_FixMissingBones_UI 
	)
	
	on btnSave pressed do (
		CreateDialog PoseTransposer_SaveNewPose_UI
	)
)
		
function AttemptLoadPose poseId = ( -- This should be in core functions, but the way MaxScript loads, it'd error when trying to create the missing-bones ui
	poseSet = poseSets[poseId]
	
	boneNames = #()
	for i=2 to poseSet.count do (
		append boneNames poseSet[i][1]
	)
	
	missingBones = GetMissingBones boneNames
	
	if missingBones.count < 1 then (
		LoadPose poseSet
		
		return true
	) else (
		activePoseSet = deepCopy poseSet
		
		CreateDialog PoseTransposer_FixMissingBones_UI
	)
	
	return false
)

rollout PoseTransposer_Main_UI "Pose Transposer" width:220 (
	listbox lstAvailablePoses "Poses"
	button btnForce "Force Pose"
	button btnLoad "Load Pose"
	button btnCreate "Create Pose"
	button btnEdit "Edit Pose"
	
	on PoseTransposer_Main_UI open do (
		lbox_availPoses = lstAvailablePoses
		
		LoadPoses()
	)
	
	on btnForce pressed do (
		hasFail = False
		
		for o in geometry do (
			if classof o == Editable_Mesh then (
				if o.modifiers[#skin] != undefined then (
					skinMod = o.modifiers[#skin]
					
					skinMod.always_deform = false
					
					origBonePos = #()
					for b in geometry do (
						append origBonePos b.position
					)
					
					if not (AttemptLoadPose lstAvailablePoses.selection) then (
						hasFail = true
					)
					
					for i=1 to geometry.count do (
						geometry[i].position = origBonePos[i]
					)
					
					skinMod.always_deform = true
				)
			)
		)
		
		if not hasFail then (
			messagebox "Pose forced successfully!"
		)
	)
	
	on btnCreate pressed do (
		createDialog PoseTransposer_Create_UI
	)
	
	on btnLoad pressed do (
		if (AttemptLoadPose lstAvailablePoses.selection) then (
			messagebox "Pose loaded successfully!"
		)
	)
)

createDialog PoseTransposer_Main_UI