-- Globals
global skyrim_morpher_importer_markedForDelete = #()
global skyrim_morpher_importer_morphBases = #()

-- Functions
function removeSubstring sourceStr subStr = (
	strLength = sourceStr.count
	substrLength = subStr.count
	strStart = findString sourceStr subStr
	
	newStr = sourceStr
	if strStart != undefined then (
		newStr = (substring sourceStr 1 (strStart - 1)) + (substring sourceStr (strStart + substrLength) strLength)
	)
	
	return newStr
)

function getFilename fPath = (
	fName = filenameFromPath fPath
	fLength = fName.count
	
	fName = substring fName 1 (fLength - 4)
	
	return fName
)

function getFolder fPath = (
	fileName = getFilename fPath
	folderName = substring fPath 1 (fPath.count - fileName.count - 4)
	
	return folderName
)

function extractTri fPath triExtractor = (
	-- Establish the folder and filenames
	fNameForFolder = getFilename fPath
	subFolder = "" + (getFolder fPath) + fNameForFolder
	fullName = subFolder + "/" + (filenameFromPath fPath)
	
	-- DosCommand is fucking stupid and can't handle spaces. Write the ACTUAL command to a batch file
	dosCmd = "C:/skyrim_importer_temp.bat"
	batchContents = "\"" + triExtractor + "\" \"" + fullName + "\""
	batchFile = createFile dosCmd
	format batchContents to:batchFile
	
	-- Move the .tri into a folder named after it, to keep the OBJs separate
	makedir subFolder
	renameFile fPath fullName
	
	-- Run the batch to run the command to run TriExtractor. This needs to wait for user input unfortunately.
	dosCommand dosCmd
	
	-- Delete the batch file
	deleteFile dosCmd
	
	-- Report how many morphs were extracted
	allObjs = getFiles (subFolder + "/*.obj")
	
	print ("Extracted " + (allObjs.count as String) + " morphs.")
	
	-- And return the OBJ files.
	return allObjs
)

function cleanTriExtraction fPath triExtractor = (
	-- Establish the folder and filenames
	fNameForFolder = getFilename fPath
	subFolder = "" + (getFolder fPath) + fNameForFolder
	fullName = subFolder + "/" + (filenameFromPath fPath)	
	
	-- Get the OBJs and the TXT file
	allObjs = getFiles (subFolder + "/*.obj")
	allTxt = getFiles (subFolder + "/*.txt")

	-- Delete them
	for file in allObjs do (
		deleteFile file
	)
	for file in allTxt do (
		deleteFile file
	)
	
	-- Move the tri back to where it was, and delete the folder
	renameFile fullName fPath
	
	deleter = DotNetClass "System.IO.Directory"
	deleter.Delete subFolder true
)
	
function batchImport arrayObjs = (
	clearSelection
	
	importedObjs = #()
	for obj in arrayObjs do (
	--for i=1 to 3 do (
		--obj = arrayObjs[i]
		
		objName = getFilename obj
		
		ImportFile obj #noPrompt
		
		selection[1].name = objName
		
		append importedObjs selection[1]
	)
	
	return importedObjs
)

function createMasterMorphers = (
	masterMorphs = #()
	
	-- Create all of the potential master Morphs
	for obj in skyrim_morpher_importer_morphBases do (
		if isValidObj obj then (
			masterMorph = copy obj
			
			masterMorph.name = "MASTER " + masterMorph.name
			
			-- Delete the old Morpher, and put a new, clean one on.
			deleteModifier masterMorph masterMorph.modifiers[#morpher]
			
			oMorph = Morpher()
			oMorph.Autoload_of_targets = 1
			
			addModifier masterMorph oMorph
			
			append masterMorphs masterMorph
		)
	)
	
	-- Iterate through all of the potential master Morphs, deleting duplicates as necessary
	for obj in masterMorphs do (
		oMorph = obj.modifiers[#morpher]
		
		-- Loop through all the other possible master morphs, to find any duplicates
		for otherObj in masterMorphs do (
			if obj != otherObj then (
				WM3_MC_BuildFromNode oMorph 1 otherObj
				if WM3_MC_HasData oMorph 1 then ( -- If otherObj can be a Morpher for obj, then they are duplicates. Delete obj and remove it from the list of master morphs.
					deleteInd = findItem masterMorphs obj
					if deleteInd > 0 then (
						deleteItem masterMorphs deleteInd
					)
					if isValidObj obj then (
						delete obj
					)
					break
				)
				WM3_MC_Delete oMorph 1
			)
		)
	)
	
	-- The master list is now clean of duplicates. Build the master morphs.
	for obj in masterMorphs do (
		if isValidObj obj then (
			mChannel = 1
			oMorph = obj.modifiers[#morpher]
			
			for otherObj in skyrim_morpher_importer_morphBases do (
				if isValidObj otherObj then (
					--if WM3_MC_IsValid oMorph mChannel then (
						print ("attempting to morph "+otherObj.name+" onto "+obj.name)
						
						WM3_MC_BuildFromNode oMorph mChannel otherObj
						if WM3_MC_HasData oMorph mChannel then ( -- Only occurs if otherObj can be a Morpher for obj
							WM3_MC_SetValue oMorph mChannel 100.0
							mChannel += 1
						)
					--)
				)
			)
		)
	)
	
	-- And then finally hide all of the master morphs
	for obj in skyrim_morpher_importer_morphBases do (
		if isValidObj obj then (
			hide obj
		)
	)
)
-- Rollout
rollout SkyrimImporter_Autonomous "Skyrim Autonomous Morph Importer" width:300 (
	button btnSelectExtractor "Select TriExtractor.exe" width:280
	edittext txtExtractorLocation "TriExtractor.exe path"
	
	label spacer ""
	
	button btnStartImport "Start Importing"
	button btnDelete "Clean Up Morph Targets"
	
	on btnDelete pressed do (
		deleteCount = 0
		
		for obj in skyrim_morpher_importer_markedForDelete do (
			if isValidObj obj then (
				delete obj
				deleteCount += 1
			)
		)
		
		skyrim_morpher_importer_markedForDelete = #()
		
		print ("Cleaned up " +(deleteCount as string)+ " objects.")
	)
	
	on btnSelectExtractor pressed do (
		triExtractor = GetOpenFileName caption:"Select TriExtractor.exe" types:"Executable (*.exe)|*.exe"
		
		if triExtractor != undefined then (
			txtExtractorLocation.text = triExtractor
		)
	)
	
	on btnStartImport pressed do (
		fileDialog = dotNetObject "System.Windows.Forms.OpenFileDialog"
		fileDialog.title = "Select all .tri you wish to convert"
		fileDialog.Multiselect = true
		
		onClose = fileDialog.showDialog()
		onClose.toString()
		
		if onClose.Equals onClose.OK then (
			files = fileDialog.fileNames

			for file in files do (
				-- Extract the morphs for this .tri
				morphObjs = extractTri file txtExtractorLocation.text
				
				-- Import all of them
				importedObjs = batchImport morphObjs
				
				-- Cull the names
				masterObj = importedObjs[1]
				importedObjs = deleteItem importedObjs 1
				
				for obj in importedObjs do (
					obj.name = removeSubstring obj.name (masterObj.name + "_")
				)
				
				-- Create as many morph bases as necessary
				numMorphers = (ceil ((importedObjs.count as float) / 100.0)) as integer
				
				morphBases = #()
				append morphBases masterObj
				for i=1 to (numMorphers-1) do (
					newBase = copy masterObj
					newBase.name = masterObj.name + " " + ((i+1) as String)
					
					append morphBases newBase
				)
				
				-- Create the Morphers
				for morphBase in morphBases do (
					morphMod = Morpher()
					morphMod.Autoload_of_targets = 1
					
					addModifier morphBase morphMod
					
					append skyrim_morpher_importer_morphBases morphBase
				)
				
				-- Load the Morph targets
				for i=1 to importedObjs.count do (
					morphChannel = (mod i 100) as integer
					morphIndex = (ceil ((i as float) / 100.0)) as integer
					morphBase = morphBases[morphIndex]
					morphMod = morphBase.modifiers[#morpher]
					
					WM3_MC_BuildFromNode morphMod morphChannel importedObjs[i]
					WM3_MC_SetName morphMod morphChannel importedObjs[i].name
					
					-- And mark them for deletion later
					append skyrim_morpher_importer_markedForDelete importedObjs[i]
				)
				
				-- Rebuild the internal cache of the Morphers
				for morphBase in morphBases do (
					morphMod = morphBase.modifiers[#morpher]
					
					WM3_RebuildInternalCache morphMod
					WM3_RefreshChannelListUI morphMod
					WM3_RefreshChannelParamsUI morphMod
				)
				
				-- Clean up after the extraction
				cleanTriExtraction file txtExtractorLocation.text
			)
			
			-- After all of the Morphers have been created, create the master morphers
			createMasterMorphers()
			
			-- And then notify it's ready for the user to clean
			messageBox "The Import sequence is complete. You may now clean up Morph Targets." beep:false
		)
	)
)

createDialog SkyrimImporter_Autonomous