global WBST_numProcessed = 0
global WBST_totalProcessing = 0
global WBST_visibility = #()

-- Helper functions
fn GetChildWindowByName winHandle childName = (
	local children = UIAccessor.GetChildWindows winHandle
	
	for o in children do (
		local childText = UIAccessor.GetWindowText o
		
		if childText == childName then (
			return o
		)
	)
	
	return undefined
)

fn SetCheckboxState winHandle boxName state = (
	-- Huge thanks to http://www.scriptspot.com/forums/3ds-max/general-scripting/press-radio-button-with-maxscript-in-command-panel for this code
	
	local BN_CLICKED = 0 -- clicky message ID
	local BM_SETCHECK = 241 -- checkbutton toggle message ID
	local WM_COMMAND = 273 -- windows command message	
	
	local chkBox = GetChildWindowByName winHandle boxName
	
	local parent = UIAccessor.getParentWindow chkBox
	local id = UIAccessor.getWindowResourceID chkBox
	
	windows.sendMessage chkBox BM_SETCHECK (if state then 1 else 0) 0
	windows.sendMessage parent WM_COMMAND ((bit.shift BN_CLICKED 16) + id) chkBox
)

fn GetLocalFilename fName = (
	dirs = filterString fname "/\\"
	
	localFname = dirs[dirs.count]
	
	ext = filterString localFname "."
	
	return ext[1]
)

function arrayHas haystack needle = (
	if finditem haystack needle == 0 then (
		return false
	)
	
	return true
)

function hideAllBut meshArray bKeepBones:true = (
	for o in Geometry do (
		bHide = true
		
		if arrayHas meshArray o then (
			bHide = false
		)
		
		if bKeepBones then (
			if classof o == BoneGeometry or classof o == Dummy then (
				bHide = false
			)
		)
		
		o.ishidden = bHide
	)
)

function unhideAll = (
	for o in Geometry do (
		if classof o == Editable_mesh or classof o == BoneGeometry or classof o == Dummy then (
			o.ishidden = false
		)
	)
)

function pushVisibility = (
	for o in Geometry do (
		append WBST_visibility #(o,o.ishidden)
	)
)

function popVisibility = (
	for aobj in WBST_visibility do (
		aobj[1].ishidden = aobj[2]
	)
	
	WBST_visibility = #()
)

function createExportGroup exportName exportMeshes = ( -- Kind of a silly function, but for posterity's sake, since some people might be antsy making these sloppy structs explicitly.
	return #(exportName, exportMeshes)
)

function prepExportGroup exportGroup bKeepBones:true = ( -- Similarly silly function
	hideAllBut exportGroup[2] bKeepBones:bkeepBones
)

-- Notification functions
fn DmImportSkeleton = (
	WinHandle = DialogMonitorOps.GetWindowHandle()
	
	WinText = UIAccessor.GetWindowText WinHandle
	
	if WinText == "Pre-parse options" then (
		UIAccessor.PressButtonByName WinHandle "OK"
	)
	if WinText == "3DS Max 2010/11 Source SMD File Importer" then (
		UIAccessor.PressButtonByName WinHandle "OK"
	)
	if WinText == "3DS Max Valve SMD File Exporter" then (
		UIAccessor.PressButtonByName WinHandle "OK"
		
		WBST_numProcessed = WBST_numProcessed + 1
		
		if WBST_numProcessed >= WBST_totalProcessing then (
			DialogMonitorOps.UnRegisterNotification id:"importer"
			
			DialogMonitorOps.Enabled = false
		)
	)
	
	return true
)

fn DmImportNoSkeleton = (
	WinHandle = DialogMonitorOps.GetWindowHandle()
	
	WinText = UIAccessor.GetWindowText WinHandle
	
	if WinText == "Pre-parse options" then (
		UIAccessor.PressButtonByName WinHandle "OK"
	)
	if WinText == "3DS Max 2010/11 Source SMD File Importer" then (
		SetCheckboxState WinHandle "Skin mesh to bones" false
		SetCheckboxState WinHandle "Import bones" false
		
		UIAccessor.PressButtonByName WinHandle "OK"
	)
	if WinText == "3DS Max Valve SMD File Exporter" then (
		UIAccessor.PressButtonByName WinHandle "OK"
		
		WBST_numProcessed = WBST_numProcessed + 1
		
		if WBST_numProcessed >= WBST_totalProcessing then (
			DialogMonitorOps.UnRegisterNotification id:"importer"
			
			DialogMonitorOps.Enabled = false
		)
	)
	
	return true
)

fn DmExport = (
	WinHandle = DialogMonitorOps.GetWindowHandle()
	
	WinText = UIAccessor.GetWindowText WinHandle
	
	if WinText == "3DS Max 2010/11 Source SMD File Exporter" then (
		SetCheckboxState WinHandle "Use explicit normals method" true
		
		UIAccessor.PressButtonByName WinHandle "OK"
	)
	
	if WinText == "3DS Max Source SMD File Exporter" then (
		UIAccessor.PressButtonByName WinHandle "OK"
		
		WBST_numProcessed = WBST_numProcessed + 1
		
		if WBST_numProcessed >= WBST_totalProcessing then (
			DialogMonitorOps.UnRegisterNotification id:"exporter"
			
			DialogMonitorOps.Enabled = false
		)
	)
	
	return true
)

-- Core functions
function ImportSMDSilent_Internal fName bUppercase bSkipSkeleton = (
	-- Call the import on the file
	ImportFile fName
	
	-- After the import, set the name of the mesh
	for o in selection do (
		if classof o == Editable_Mesh then (
			local oName = GetLocalFilename fName
			
			-- If toUpper, then "my_mesh" -> "MY MESH"
			if bUppercase then (
				oName = substituteString oName "_" " "
				oName = toUpper oName
			)
			
			o.name = oName
		)
	)
)

function ImportSMDSilent_Preamble fNames bUppercase bSkipSkeleton = (
	WBST_numProcessed = 0
	WBST_totalProcessing = fNames.count
	
	-- Set up the dialog automation.
	DialogMonitorOps.Enabled = true
	
	if bSkipSkeleton then (
		DialogMonitorOps.RegisterNotification DmImportNoSkeleton id:"importer"
	) else (
		DialogMonitorOps.RegisterNotification DmImportSkeleton id:"importer"
	)
)

function ImportSMDSilent fNames bUppercase:false bSkipSkeleton:false = (
	ImportSMDSilent_Preamble fNames bUppercase bSkipSkeleton
	
	for fName in fNames do (
		ImportSMDSilent_Internal fName bUppercase bSkipSkeleton
	)
)

function ExportSMDSilent_Preamble exportGroups = (
	WBST_numProcessed = 0
	WBST_totalProcessing = exportGroups.count
	
	DialogMonitorOps.Enabled = true
	
	DialogMonitorOps.RegisterNotification DmExport id:"exporter"
)

function ExportSMDSilent_Internal exportGroup bKeepBones:true = (
	pushVisibility()
	
	prepExportGroup exportGroup bKeepBones:bKeepBones
	
	ExportFile exportGroup[1]
	
	popVisibility()
)

function ExportSMDSilentBatch exportGroups bKeepBones:true = (
	ExportSMDSilent_Preamble exportGroups
	
	for exportGroup in exportGroups do (
		ExportSMDSilent_Internal exportGroup bKeepBones:bKeepBones
	)
)

function ExportSMDSilent fName = ( -- Special, "simple" export. Just creates an export group of the current non-hidden meshes.
	allMeshes = for o in selection where ((classof o == Editable_Mesh or classof o == Editable_Poly) and o.ishidden == false) collect o
		
	exportGroup = createExportGroup fName allMeshes
	exportGroups = #(exportGroup)
	
	ExportSMDSilentBatch exportGroups bKeepBones:true
)