WT_BoneMatches = #()

rollout RobotRemoveBone "Replace Bone" width:204
(
	dropDownList ddl1 "Remove This Bone:" pos:[8,8] width:165 height:40
	button btnCurrentUseBone1 "*" pos:[174,24] width:24 height:24 toolTip:"Selected Bone"
	dropDownList ddl2 "Assign Verts to this Bone:" pos:[8,64] width:165 height:40
	button btnCurrentUseBone2 "*" pos:[174,80] width:24 height:24 toolTip:"Selected Bone"
	button btn1 "Replace Bone" pos:[112,128] width:80 height:24
	pickbutton btn2 "Pick Obj" pos:[22,128] width:80 height:24
	
	edittext txtMatcher "Pattern: To Remove" pos:[8,180]
	edittext txtReplacer "Pattern: To Copy To" pos:[8, 204] tooltip:"Replace the Matcher string with this to find the matched pair. Leave blank if the Matcher is an appending."
	button btnPopulateMatches "Find Matches" pos:[8,230] width:190
	
	label lblCnts "Current bone / total bones:" pos:[8,260]
	edittext txtCurrentBone "" pos:[140,260] width:24 enabled:false
	edittext txtMaxBones "" pos:[164,260] width:24 enabled:false
	
	button btnOldToNew "Iterate Replacement" pos:[8,290] width:190 tooltip:"Replace the current bone. Press this until all bones are replaced."
	
	button btnRemoveDeadBones "Remove Dead Bones" pos:[8,314] width:190
	button btnSelectDeadBones "Select Dead Bones" pos:[8,338] width:190
	
	fn populateDDL obj =
	(
		boneArray = #()
		for i = 1 to skinOps.GetNumberBones obj.skin do
		(
			append boneArray (skinOps.GetBoneName obj.skin i 1)
		)

		ddl1.items = boneArray
		ddl2.items = boneArray
	)
	
	fn FindMatchedBones obj matcherString replacerString = (
		bonePairs = #()
		
		numBones = skinOps.GetNumberBones obj.skin
		
		for i=1 to numBones do (
			boneName = skinOps.getBoneName obj.skin i 1
			
			if findString boneName matcherString != undefined then ( -- This bone contains the matcher. It could possibly be paired, then.				
				partnerName = substituteString boneName matcherString replacerString
				
				for j=1 to numBones do (
					pBoneName = skinOps.getBoneName obj.skin j 1
					
					if pBoneName == partnerName then ( -- this is the partner bone!!
						append bonePairs #(i,j) -- Add the bone pair, in the form of (<matcher>,<replacer>)
						
						exit -- Stop this inner loop; we found the partner
					)
				)
			)
		)
		
		return bonePairs
	)
	
	fn RemoveBonesFromSkin obj bonesList = (
		fn DescendingSort a b = (
			return b - a
		)
		
		sortedBonesList = deepcopy bonesList
		qsort sortedBonesList DescendingSort
		
		for i in sortedBonesList do (
			skinOps.removeBone obj.skin i
		)
	)
	
	fn FindDeadBones obj = (
		allWeightedBones = #()
		
		numBones =  skinOps.getNumberBones obj.skin
		
		-- First, create an array mapping bone IDs to the number of times they're weighted
		for i=1 to numBones do (
			allWeightedBones[i] = 0
		)
		
		-- Then, go through every vert in the skin
		for vert=1 to skinOps.getNumberVertices obj.skin do (
			
			numWeightedBones = skinOps.getVertexWeightCount obj.skin vert
			
			-- And for every vert, every bone that has weight assigned to it
			for weightedBone=1 to numWeightedBones do (
				actualBoneId = skinOps.getVertexWeightBoneID obj.skin vert weightedBone -- Get the actual bone ID
				boneWeight = skinOps.getVertexWeight obj.skin vert weightedBone -- and the weight
				
				-- Account for some stupid decimal imprecisions (like 0.00000000001)
				if (boneWeight - 0.001) > 0 then ( -- If this is a non-trivial weight, then count it toward this bone
					allWeightedBones[actualBoneId] = allWeightedBones[actualBoneId] + 1
				)
			)
		)
		
		-- Once every vert has been operated on, find all the bones with no weights
		deadBones = #()
		
		for i=1 to numBones do (
			if allWeightedBones[i] < 1 then (
				append deadBones i
			)
		)
		
		return deadBones
	)
	
	fn FindDeadBoneNames obj = (
		allWeightedBones = #()
		allBoneNames = #()
		
		numBones =  skinOps.getNumberBones obj.skin
		
		-- First, create an array mapping bone IDs to the number of times they're weighted
		for i=1 to numBones do (
			allWeightedBones[i] = 0
		)
		
		-- Then, go through every vert in the skin
		for vert=1 to skinOps.getNumberVertices obj.skin do (
			
			numWeightedBones = skinOps.getVertexWeightCount obj.skin vert
			
			-- And for every vert, every bone that has weight assigned to it
			for weightedBone=1 to numWeightedBones do (
				actualBoneId = skinOps.getVertexWeightBoneID obj.skin vert weightedBone -- Get the actual bone ID
				boneWeight = skinOps.getVertexWeight obj.skin vert weightedBone -- and the weight
				
				allBoneNames[actualBoneId] = skinOps.getBoneName obj.skin weightedBone 0
				
				-- Account for some stupid decimal imprecisions (like 0.00000000001)
				if (boneWeight - 0.001) > 0 then ( -- If this is a non-trivial weight, then count it toward this bone
					allWeightedBones[actualBoneId] = allWeightedBones[actualBoneId] + 1
				)
			)
		)
		
		-- Once every vert has been operated on, find all the bones with no weights
		deadBones = #()
		
		for i=1 to numBones do (
			if allWeightedBones[i] < 1 then (
				append deadBones i
			)
		)
		
		deadBoneNames = #()
		
		for deadBone in deadBones do (
			append deadBoneNames allBoneNames[deadBone]
		)
		
		return deadBoneNames
	)
	
	fn selectElementsByName arrayNames = (
		selectObjs = #()
		
		-- Woo n^2 operations!
		
		for o in Geometry do (
			if classof o == BoneGeometry or classof o == Dummy then (
				for bName in arrayNames do (
					if o.name == bName then (
						append selectObjs o
						exit
					)
				)
			)
		)
		
		select selectObjs
	)
	
	fn replaceBone obj oldBone newBone = (
		if obj != undefined and OldBone != NewBone do
		(
			
			try (skinOps.GetNumberVertices obj.skin) catch (
				messagebox "Error!"
				return null
			)
			
			for v = 1 to skinOps.GetNumberVertices obj.skin do
			(
				numBones = skinOps.GetVertexWeightCount obj.skin v
				
				for w = 1 to numBones do
				(
					if OldBone == skinOps.GetVertexWeightBoneID obj.skin v w do
					(
						BoneIDArray = for bwa = 1 to numBones collect (skinOps.GetVertexWeightBoneID obj.skin v bwa)
						WeightArray = for wa = 1 to numBones collect (skinOps.GetVertexWeight obj.skin v wa)
						
						OldPlace = findItem BoneIDArray OldBone
						NewPlace = findItem BoneIDArray NewBone
						
						if NewPlace != 0 then
						(
							NewWeight = WeightArray[OldPlace] + WeightArray[NewPlace]
							
							WeightArray[OldPlace] = NewWeight
							BoneIDArray[OldPlace] = NewBone
							
							deleteItem WeightArray NewPlace
							deleteItem BoneIDArray NewPlace
						)
						else
						(
							BoneIDArray[OldPlace] = NewBone
						)
						
-- 						format "VERTEX:%\n" v
-- 						format "BoneIDArray: %\n" BoneIDArray as string
-- 						format "WeightArray: %\n" WeightArray as string
						
 						skinOps.ReplaceVertexWeights obj.skin v BoneIDArray WeightArray

						exit
					)
				)
			)
		)	
	)
	
	on RobotRemoveBone open do
	(
		if selection.count == 1 do
		(
			if isProperty selection[1] "Skin" then
			(
				--clearselection()
				--select obj
				max modify mode
				btn2.object = selection[1]
				populateDDL selection[1]
			)
		)
	)
	
	on btnCurrentUseBone1 pressed do
	(
		if selection.count == 1 do
		(
			if isProperty selection[1] "Skin" then
			(
				ddl1.selection = skinOps.GetSelectedBone selection[1].skin
			)
		)
	)
	
	on btnCurrentUseBone2 pressed do
	(
		if selection.count == 1 do
		(
			if isProperty selection[1] "Skin" then
			(
				ddl2.selection = skinOps.GetSelectedBone selection[1].skin
			)
		)
	)
	
	on btn2 picked obj do
	(
		if isProperty obj "Skin" then
		(
			--clearselection()
			select obj
			max modify mode
			populateDDL obj
		)
		else
		(
			messageBox "Please select a model with a Skin modifier."
		)
	)
	
	on btn1 pressed do
	(
		clearlistener()
		obj = btn2.object
		OldBone = ddl1.selection
		NewBone = ddl2.selection
		
		print ("OldBone = " + (OldBone as string))
		print ("NewBone = " + (NewBone as string))
			
		replaceBone obj oldBone newBone
			
		skinOps.removebone obj.skin OldBone
		populateDDL obj
	)
	
	on btnOldToNew pressed do (
		clearlistener()
		obj = btn2.object
		
		bonePairs = WT_BoneMatches
		
		if bonePairs.count < 1 then (
			messagebox "No matches to replace!"
			return null
		)
		
		curPair = txtCurrentBone.text as number
		
		bonePair = bonePairs[curPair]
			
		print ("OldBone = " + (bonePair[1] as string) + "(\"" + (skinOps.getBoneName obj.skin bonePair[1] 1) + "\")")
		print ("NewBone = " + (bonePair[2] as string) + "(\"" + (skinOps.getBoneName obj.skin bonePair[2] 1) + "\")")
		print "-----"
			
		replaceBone obj bonePair[1] bonePair[2]
		
		curPair = curPair + 1
		
		txtCurrentBone.text = curPair as string
		
		if curPair >(txtMaxBones.text as number) then (
			messageBox "All bones replaced" beep:false
			
			txtCurrentBone.text = ""
			txtMaxBones.text = ""
			WT_BoneMatches = #()
		)
	)
	
	on btnPopulateMatches pressed do (
		WT_BoneMatches = FindMatchedBones btn2.object txtMatcher.text txtReplacer.text
		
		txtCurrentBone.text = "1"
		txtMaxBones.text = WT_BoneMatches.count as string
	)
	
	on btnRemoveDeadBones pressed do (
		deadBones = FindDeadBones btn2.object
		
		RemoveBonesFromSkin btn2.object deadBones
	)
)

createDialog RobotRemoveBone